@@ -24,6 +24,7 @@
 
 
 import sys
+import re
 import os
 from datetime import datetime
 import bsddb
@@ -39,6 +40,7 @@ except:
 from lib.lib import banner_notification, list_presence_users
 from lib.server import Bonjour_server
 from lib.sms_listener import Sms_listener
+from lib.call_listener import Call_listener
 from lib.scheduler import Scheduler
 from lib.lib import logger
 from lib.friend import Friend
@@ -91,6 +93,9 @@ class Central_widget(QtGui.QWidget):
         self.parent.main_window.setAttribute(
                                     QtCore.Qt.WA_Maemo5ShowProgressIndicator,
                                     True)
+        # Read last bonjour_contact used
+        last_authorized_bonjour_contact = config.read_last_authorized_bonjour_contact()
+
         self.parent.main_window.repaint()
         banner_notification("Looking for Bonjour contacts ...")
         self.parent.bonjour_users = list_presence_users()
@@ -103,11 +108,10 @@ class Central_widget(QtGui.QWidget):
             for bonjour_user, info in self.parent.bonjour_users.items():
                 # if the contact was create by heysms
                 # we don't want to see it in the list
+                bonjour_user = bonjour_user.decode('utf-8')
                 if bonjour_user in node_friend_list:
-                        continue
+                    continue
                 self.bonjour_users.addItem(bonjour_user)
-            # Read last bonjour_contact used
-            last_authorized_bonjour_contact = config.read_last_authorized_bonjour_contact()
             # Search in list last bonjour_contact used
             index = self.bonjour_users.findText(last_authorized_bonjour_contact)
             if index != -1:
@@ -115,7 +119,6 @@ class Central_widget(QtGui.QWidget):
                 logger.debug("Last Bonjour contact found: %s, we select it" % last_authorized_bonjour_contact)
                 self.bonjour_users.setCurrentIndex(index)
 
-            self.parent.change_bonjour_user(self.bonjour_users.currentText())
             banner_notification("Bonjour contacts loaded !")
 
         self.parent.main_window.setAttribute(
@@ -142,8 +145,10 @@ class Ui_MainWindow(QtCore.QObject):
         self.central_widget = Central_widget(self)
         self.main_window.setCentralWidget(self.central_widget)
         self.main_window.conf_dialog = Config_dialog(self.main_window)
-        # Set ui
+        # Load saved config
         config.init_profile()
+        if config.useusb == QtCore.Qt.Checked:
+            config.start_useusb()
 
         ### Menu
         self.menubar = MenuBar(self.main_window)
@@ -181,10 +186,7 @@ class Ui_MainWindow(QtCore.QObject):
             self.bonjour_auth_user = self.central_widget.bonjour_users\
                                          .currentText()
             self.bs = Bonjour_server(self.bonjour_auth_user)
-        if self.bs.is_running():
-            pass
-#            self.bs.stop()
-        else:
+        if not self.bs.is_running():
             self.bs.listen()
 
     def change_bonjour_user(self, new_bonjour_user):
@@ -201,39 +203,58 @@ class Ui_MainWindow(QtCore.QObject):
         friends_dialog.setComboBoxEditable(0)
         db = bsddb.hashopen('/home/user/.osso-abook/db/addressbook.db', 'r')
         friend_list = {}
+        old_friend_list = [f.fullname for f in self.scheduler.friend_list]
+        q_friend_dict = {}
         for contact in db.values():
-            logger.debug("Trying to find name and "
-                         "cell number in: %s" % contact)
-            tmp = contact.split("FN:")
-            if len(tmp) > 1:
-                tmp = tmp[1].split("\r\n")
-                if len(tmp) > 1:
-                    name = tmp[0].decode('utf-8')
-            tmp = contact.split("CELL:")
-            num = None
-            if len(tmp) > 1:
-                tmp = tmp[1].split("\r\n")
-                if len(tmp) > 1:
-                    num = tmp[0]
-            i = 1
-            while name in friend_list:
-                name = name + " (%s)" % str(i)
-                i = i + 1
-
-            if num is None:
-                logger.debug("friend: %s has not cell number")
+#            logger.debug("Trying to find name and "
+#                         "cell number in: %s" % contact.split('\n'))
+            reg = re.compile('FN:(.*?)\\r', re.S|re.M)
+            numbers = re.findall("TEL;TYPE=(?:HOME,|WORK,|)CELL:(.[0-9]*)\r\n", contact)
+            m = re.search(reg, contact)
+            if m is None:
+                logger.debug("No name in contact")
+                continue
+            elif not numbers:
+                logger.debug("No cell phone found")
                 continue
 
-            friend_list[name] = num
+            name = m.group(1).decode('utf-8')
+            for num in numbers:
+                logger.debug("Found : %s - %s" % (name, num))
+                if name in old_friend_list:
+                    logger.debug("Friend already in list: %s" % name)
+                    continue
 
-        q_friend_list = QtCore.QStringList(sorted(friend_list.keys()))
+                i = 1
+                while name in friend_list:
+                    name = name + " (%s)" % str(i)
+                    i = i + 1
+
+                # Prepare lists
+                friend_list[num] = name
+                q_friend_dict["%s - %s" % (name, num)] = num
+
+        q_friend_list = QtCore.QStringList(sorted(q_friend_dict.keys()))
         friends_dialog.setComboBoxItems(q_friend_list)
         friends_dialog.setWindowTitle("Add a friend")
         friends_dialog.setLabelText("Select a friend")
-        friends_dialog.exec_()
-        select_friend = unicode(friends_dialog.textValue())
-        number = friend_list[select_friend]
-        auth_user = self.bonjour_auth_user
+        ret = friends_dialog.exec_()
+
+        if ret == 0:
+            logger.debug("Dialog not confirmed")
+            return
+
+        raw_name = str(friends_dialog.textValue())
+        number = q_friend_dict[raw_name]
+        select_friend = friend_list[number]
+        bonjour_auth_username = str(self.bonjour_auth_user)
+        auth_user = {bonjour_auth_username:
+                self.bonjour_users[bonjour_auth_username]}
+
+        # Check if friend is already in friend list
+        if select_friend in old_friend_list:
+                logger.debug("Selected friend already in list: %s" % select_friend)
+                return
 
         new_friend = Friend(select_friend, number, auth_user)
         # Add to friend list in table model
@@ -247,7 +268,7 @@ class Ui_MainWindow(QtCore.QObject):
         icon = QtGui.QIcon('/usr/share/icons/hicolor/48x48/hildon'
                            '/general_refresh.png')
         vars = {}
-        vars['version'] = "1.3.0"
+        vars['version'] = "1.6.1"
         message = QtCore.QString("""<center><h2>HeySms</h2></center>"""
                                  """<center>Version: %(version)s</center>"""
                                  """<br/>Visite web site: <a href="http://talk.maemo.org/showthread.php?t=84705">Here</a> """
@@ -268,8 +289,16 @@ def main():
         logger.debug("Stopping active friends")
         for f in ui.friend_list:
             f.close()
+        logger.debug("Shutdown scheduler")
+        ui.scheduler.must_run = False
+        while ui.scheduler.isRunning():
+            logger.debug("Waiting scheduler")
+            sleep(0.1)
         logger.debug("Restore profile")
         config.restore_profile()
+        if config.useusb == QtCore.Qt.Checked:
+            logger.debug("Restore USB")
+            config.restore_useusb()
 
     opt_parser = OptionParser()
     opt_parser.add_option("-d", "--debug", dest="debug_mode",
@@ -288,6 +317,9 @@ def main():
     ui.setupUi(main_window)
     config.parent = ui
 
+    ui.call_listener = Call_listener(ui)
+    ui.call_listener.start()
+    logger.debug("Call_listener started")
     ui.sms_listener = Sms_listener(ui)
     ui.sms_listener.start()
     logger.debug("Sms_listener started")
@@ -304,7 +336,7 @@ def main():
 
 
     QtCore.QObject.connect(app,
-                           QtCore.SIGNAL("lastWindowClosed()"),
+                           QtCore.SIGNAL("aboutToQuit()"),
                            exit)
 
     sys.exit(app.exec_())
